-- NoRelease --

--[[
TODO:

PROBLEMS:
--]]

local DefaultO = {
  ["framePoint"] = "CENTER";
  ["frameRelativeTo"] = "UIParent";
  ["frameRelativePoint"] = "CENTER";
  ["frameOffsetX"] = 0;
  ["frameOffsetY"] = 0;
  ["scaleFrame"] = 1;
  ["encounterIDs"] = {};
}

--split string containing quoted and not quoted arguments
--input pattern: (\S+|".+")?(\s+(\S+|".+"))*
--example input: [[arg1 "arg2part1 arg2part2" arg3]]
--example output: {"arg1", "arg2part1 arg2part2", "arg3"}
local function mysplit2(inputstr)
  local i, i1, i2, l, ret, retI = 1, 0, 0, inputstr:len(), {}, 1
  --remove leading spaces
  i1, i2 = inputstr:find("^%s+")
  if i1 then
    i = i2 + 1
  end
  
  while i <= l do
    --find end of current arg
    if (inputstr:sub(i, i)) == "\"" then
      --quoted arg, find end quote
      i1, i2 = inputstr:find("\"%s+", i + 1)
      if i1 then
        --spaces after end quote, more args to follow
        ret[retI] = inputstr:sub(i + 1, i1 - 1)
        retI = retI + 1
        i = i2 + 1
      else
        i1, i2 = inputstr:find("\"$", i + 1)
        if i1 then
          --end of msg
          ret[retI] = inputstr:sub(i + 1, i1 - 1)
          return ret
        else
          -- no end quote found, or end quote followed by no-space-charater found, disregard last arg
          return ret
        end
      end
    else
      --not quoted arg, find next space (if any)
      i1, i2 = inputstr:find("%s+", i + 1)
      if i1 then
        --spaces after arg, more args to follow
        ret[retI] = inputstr:sub(i, i1 - 1)
        retI = retI + 1
        i = i2 + 1
      else
        --end of msg
        ret[retI] = inputstr:sub(i)
        return ret
      end
    end
  end
  return ret
end

local updateOptionsArray, cloneTable, copyValues
--from "SCT" by "Grayhoof"
cloneTable = function(t) -- return a copy of the table t
  local new = {}; -- create a new table
  local i, v = next(t, nil); -- i is an index of t, v = t[i]
  while i do
    if type(v)=="table" then
      v=cloneTable(v);
    end
    new[i] = v;
    i, v = next(t, i); -- get next index
  end
  return new;
end
copyValues = function(from,to)
  local i, v = next(from, nil); -- i is an index of from, v = from[i]
  while i do
    if type(v)=="table" then
      if to[i] ~= nil then
        if type(to[i])=="table" then
          copyValues(v,to[i]);
        end
      end
    else
      if to[i] ~= nil then
        if type(to[i])~="table" then
          to[i] = v;
        end
      end
    end
    i, v = next(from, i); -- get next index
  end
end
updateOptionsArray = function()
  --copy default keys
  local TempNoReleaseOptions = cloneTable(DefaultO)
  copyValues(NoReleaseOptions, TempNoReleaseOptions)
  
  --copy back to savedvar
  NoReleaseOptions = TempNoReleaseOptions
end

local WHITE_TEXT_COLOR = {1, 1, 1, 1}
local releaseLocked = false
local currentEncounterID = 0

local frame = CreateFrame("Frame", "NoReleaseFrame", UIParent)

frame:SetFrameStrata("DIALOG")
--frame:SetMovable(true)
--frame:EnableMouse(true)
--frame:RegisterForDrag("LeftButton")

local frameEvents = {};

local isDebug = false
local function debugLog(...)
  if isDebug then
    print("NoRelease", ...)
  end
end
function NoRelease_Debug(msg, i)
  if msg == "debug" then
    isDebug = not isDebug
    print("NoRelease: debug", isDebug)
  elseif msg == "lock" then
    releaseLocked = not releaseLocked
    print("NoRelease: releaseLocked", releaseLocked)
  elseif msg == "endcombat" then
    frameEvents["ENCOUNTER_END"](frame, 1962, "Guarm", 16, 20, 0)
  end
end

frame.bgtexture = frame:CreateTexture(nil, "OVERLAY")
frame.bgtexture:SetAllPoints(frame)
frame.bgtexture:SetColorTexture(0, 0, 0, 1)

frame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
frame:SetScript("OnDragStop", function(self)
  self:StopMovingOrSizing()
  local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint()
  NoReleaseOptions["framePoint"] = point or DefaultO["framePoint"]
  NoReleaseOptions["frameRelativeTo"] = relativeTo or DefaultO["frameRelativeTo"]
  NoReleaseOptions["frameRelativePoint"] = relativePoint or DefaultO["frameRelativePoint"]
  NoReleaseOptions["frameOffsetX"] = xOfs or DefaultO["frameOffsetX"]
  NoReleaseOptions["frameOffsetY"] = yOfs or DefaultO["frameOffsetY"]
end);

local function initFrame()
  frame:SetPoint(NoReleaseOptions["framePoint"], NoReleaseOptions["frameRelativeTo"], NoReleaseOptions["frameRelativePoint"], NoReleaseOptions["frameOffsetX"], NoReleaseOptions["frameOffsetY"])
  frame:SetFrameStrata("LOW")
  frame:SetSize(260, 42)
  
  frame.scaleFrames = function(self, scale, doNotSet)
    if not doNotSet then
      NoReleaseOptions["scaleFrame"] = scale
    end
    self:SetScale(scale)
  end
  frame:scaleFrames(NoReleaseOptions["scaleFrame"], true)
  
  local qstring = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
  qstring:SetPoint("TOP", frame, "TOP", 0, -2)
  qstring:SetTextColor(unpack(WHITE_TEXT_COLOR))
  qstring:SetText("Type \"RELEASE\" to enable release option:")
  frame.qstring = qstring
  
  local nrtextbox = CreateFrame("EditBox", nil, frame, "InputBoxTemplate")
  frame.nrtextbox = nrtextbox
  nrtextbox:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 8, 2)
  nrtextbox:SetAutoFocus(false)
  nrtextbox:SetWidth(100)
  nrtextbox:SetHeight(20)
  nrtextbox:SetTextInsets(0,0,3,3)
  nrtextbox:SetText("")
  nrtextbox:SetCursorPosition(0)
  nrtextbox:SetScript("OnEnterPressed", function(self)
    self:ClearFocus()
    if not releaseLocked then
      if StaticPopup_Visible("DEATH") then
        for index = 1, STATICPOPUP_NUMDIALOGS, 1 do
          local frame = _G["StaticPopup"..index];
          if ( frame:IsShown() and (frame.which == "DEATH") ) then
            frame.button1:Click()
            break
          end
        end
      end
    end
  end);
  nrtextbox:SetScript("OnEscapePressed", function(self)
    self:ClearFocus()
  end);
  nrtextbox:SetScript("OnTextChanged", function(self, userInput)
    releaseLocked = not (string.lower(self:GetText()) == "release")
  end);
  
  frame.nrcheck = CreateFrame("CheckButton", "NoReleaseFrameCheckBox", frame, "ChatConfigCheckButtonTemplate")
  frame.nrcheck.parent = frame
  frame.nrcheck:SetPoint("BOTTOMLEFT", frame.nrtextbox, "BOTTOMRIGHT", 4, 0)
  frame.nrcheck.text = _G["NoReleaseFrameCheckBoxText"]
  frame.nrcheck.text:SetText("Lock: ???")
  frame.nrcheck.tooltip = "Always require user input for this encounter"
  
  frame.nrcheck:SetChecked(false)
  frame.nrcheck:SetScript("OnClick", function(self)
    if currentEncounterID > 0 then
      for _, v in pairs(NoReleaseOptions.encounterIDs) do
        if v.id == currentEncounterID then
          if self:GetChecked() then
            v.lock = 1
            self.parent.nrtextbox:SetText("")
          else
            v.lock = 0
            self.parent.nrtextbox:SetText("RELEASE")
          end
          break
        end
      end
    end
  end)
  frame.nrcheck:Hide()
  
  frame:Hide()
end

function frameEvents:ENCOUNTER_END(encounterID, encounterName, difficultyID, raidSize, endStatus)
  debugLog("encounter ended:", encounterID, encounterName, difficultyID, raidSize, endStatus)
  currentEncounterID = encounterID
  local eid = nil
  for _, v in pairs(NoReleaseOptions.encounterIDs) do
    if v.id == currentEncounterID then
      eid = v
      break
    end
  end
  if not eid then
    eid = {id = encounterID, name = encounterName, lock = 0};
    table.insert(NoReleaseOptions.encounterIDs, eid)
  end
  if StaticPopup_Visible("DEATH") then
    for index = 1, STATICPOPUP_NUMDIALOGS, 1 do
			local frame = _G["StaticPopup"..index];
			if ( frame:IsShown() and (frame.which == "DEATH") ) then
        self:Show()
        --[[
        currentEncounterID = encounterID
        local eid = nil
        for _, v in pairs(NoReleaseOptions.encounterIDs) do
          if v.id == currentEncounterID then
            eid = v
            break
          end
        end
        if not eid then
          eid = {id = encounterID, name = encounterName, lock = 0};
          table.insert(NoReleaseOptions.encounterIDs, eid)
        end
        --]]
        if eid.lock == 1 then
          self.nrcheck:SetChecked(true)
          self.nrtextbox:SetText("")
        else
          self.nrcheck:SetChecked(false)
          self.nrtextbox:SetText("RELEASE")
        end
        self.nrcheck.text:SetText("Lock: "..encounterName)
        self.nrcheck:Show()
      end
    end
  end
end

--[[
local function StaticPopup_Show_Hook(which, text_arg1, text_arg2, data, insertedFrame)
  if which == "DEATH" then
    frame:Show()
    frame.nrtextbox:SetText("RELEASE")
  end
end
--]]
local function StaticPopup_Hide_Hook(which, data)
  if which == "DEATH" then
    frame:Hide()
    frame.nrcheck:Hide()
    currentEncounterID = 0
    frame.nrtextbox:SetText("")
  end
end
local function StaticPopup_OnUpdate_Hook(dialog, elapsed)
  if dialog.which == "DEATH" then
    if releaseLocked and not IsEncounterInProgress() then
      dialog.button1:Disable()
    end
  end
end

function frameEvents:PLAYER_ENTERING_WORLD(...)
  frame:UnregisterEvent("PLAYER_ENTERING_WORLD")
  
  if not NoReleaseOptions then
    NoReleaseOptions = DefaultO
  end
  initFrame()
  
  frame:RegisterEvent("ENCOUNTER_END")
  
  --hooksecurefunc("StaticPopup_Show", StaticPopup_Show_Hook)
  hooksecurefunc("StaticPopup_Hide", StaticPopup_Hide_Hook)
  hooksecurefunc("StaticPopup_OnUpdate", StaticPopup_OnUpdate_Hook)
  
  --[[
  if StaticPopup_Visible("DEATH") then
    frame:Show()
    frame.nrtextbox:SetText("RELEASE")
  end
  --]]
end

frame:SetScript("OnEvent", function(self, event, ...)
  frameEvents[event](self, ...)
end);
frame:RegisterEvent("PLAYER_ENTERING_WORLD")

SLASH_NORELEASE1 = "/norelease"
SlashCmdList["NORELEASE"] = function(msg, editbox)
  local args = mysplit2(msg or "")
  local arg1 = string.lower(args[1] or "")
  
  --[[
  if arg1 == "move" then
    if moving then
      moving = false
      frame:SetMovable(false) --click-through
      frame:EnableMouse(false)
      frame:RegisterForDrag("")
      frame.bgtexture:SetColorTexture(0, 0, 0, 0.8)
    else
      moving = true
      frame:SetMovable(true);
      frame:EnableMouse(true);
      frame:RegisterForDrag("LeftButton");
      frame.bgtexture:SetColorTexture(0.5, 0.5, 0.5, 0.8)
    end
  else
  --]]
  if arg1 == "reset" then
    NoReleaseOptions["framePoint"] = DefaultO["framePoint"]
    NoReleaseOptions["frameRelativeTo"] = DefaultO["frameRelativeTo"]
    NoReleaseOptions["frameRelativePoint"] = DefaultO["frameRelativePoint"]
    NoReleaseOptions["frameOffsetX"] = DefaultO["frameOffsetX"]
    NoReleaseOptions["frameOffsetY"] = DefaultO["frameOffsetY"]
    
    frame:ClearAllPoints()
    frame:SetPoint(NoReleaseOptions["framePoint"], NoReleaseOptions["frameRelativeTo"], NoReleaseOptions["frameRelativePoint"], NoReleaseOptions["frameOffsetX"], NoReleaseOptions["frameOffsetY"])
    
    print("|cffaaaaffNoRelease |rposition reset")
  elseif arg1 == "scale" then
    if args[2] then
      frame:scaleFrames(tonumber(args[2]) or NoReleaseOptions["scaleFrame"])
    else
      frame:scaleFrames(NoReleaseOptions["scaleFrame"], true)
    end
    print("|cffaaaaffNoRelease |rscale:", NoReleaseOptions["scaleFrame"])
  else
    print("|cffaaaaffNoRelease |r"..(GetAddOnMetadata("NoRelease", "Version") or "").." |cffaaaaff(use |r/norelease <option> |cffaaaafffor these options)")
    --print("  move |cffaaaafftoggle moving the frame ("..(moving and "|cffaaffaamoving" or "|cffff8888fixed").."|cffaaaaff)")
    print("  reset |cffaaaaffreset the frame's position")
    print("  scale "..NoReleaseOptions["scaleFrame"].." |cffaaaaffchange the frame's scale")
  end
end